package com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v1.vo;

import com.fasterxml.jackson.annotation.JsonFormat;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import org.springframework.format.annotation.DateTimeFormat;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;
import javax.validation.constraints.NotNull;
import java.util.Date;
import java.util.List;

/**
 * 提交工作流工单Vo
 *
 * @author mazq
 * @date 2021/2/19
 */
@ApiModel(value = "提交工作流申请信息Vo")
@Data
public class ServiceAuthFlowAcceptVo {

    /**
     * 流程类型
     */
    @ApiModelProperty(value = "流程类型")
    private String processType;
    /**
     * 工单号
     */
    @ApiModelProperty(value = "申请单号")
    private String applicantId;


    /**
     * 流程标识码
     */
    @ApiModelProperty(value = "流程标识码")
    private String businessCode;

    /**
     * 申请理由
     */
    @ApiModelProperty(value = "申请理由")
    @NotBlank
    private String applyReason;

    /**
     * 申请人身份证号
     */
    @ApiModelProperty(value = "申请人身份证号")
    @NotBlank
    private String applicantIdcard;
    /**
     * 申请人姓名
     */
    @ApiModelProperty(value = "申请人姓名")
    @NotBlank
    private String applicantName;

    /**
     * 申请机构code
     */
    @ApiModelProperty(value = "申请人单位编码")
    @NotBlank
    private String applicantOrgCode;
    /**
     * 申请机构名称
     */
    @ApiModelProperty(value = "申请人单位名称")
    @NotBlank
    private String applicantOrgName;

    /**
     * 申请人联系方式
     */
    @ApiModelProperty(value = "申请人联系方式")
    @NotBlank
    private String applicantPhoneNo;

    /**
     * 权限有效类型（01：自定义 | 02：长期）
     */
    @ApiModelProperty(value = "权限有效类型")
    @NotBlank
    private String permissionValidType;

    @ApiModelProperty(value = "权限有效起始时间（2020-01-01 00:00:00）")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date startTime;

    @ApiModelProperty(value = "权限有效终止时间（2020-01-01 00:00:00）")
    @JsonFormat(timezone = "GMT+8", pattern = "yyyy-MM-dd")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    private Date endTime;

    @ApiModelProperty(value = "服务共享方式")
    private String fwgxfs;

    /**
     * 访问资源信息
     */
    @ApiModelProperty(value = "访问资源信息实体类")
    @Valid
    @NotNull
    private ResourceInfoVo resourceInfo;

    /**
     * 工作单类型
     * SERVICE_AUTH-服务授权工作单 SERVICE_CANCEL-服务权限撤销工作单
     */
    @ApiModelProperty(value = "工作单类型 SERVICE_AUTH-服务授权工作单 SERVICE_CANCEL-服务权限撤销工作单")
    private String flowType;


    @ApiModelProperty(value = "下一任务对象信息")
    private List<ActivitiHolderVo> activitiHolderList;
}
