package com.dragoninfo.dcuc.authweb.restcontroller.api.controller;

import com.dragoninfo.dcuc.app.entity.ApplyInfo;
import com.dragoninfo.dcuc.app.facade.IApplyInfoFacade;
import com.dragoninfo.dcuc.auth.auth.dto.AppFunInfoDTO;
import com.dragoninfo.dcuc.auth.auth.dto.AppResourcesDto;
import com.dragoninfo.dcuc.auth.auth.dto.ApprovalDto;
import com.dragoninfo.dcuc.auth.auth.dto.RoleApiDto;
import com.dragoninfo.dcuc.auth.auth.facade.IApprovalFacade;
import com.dragoninfo.dcuc.auth.auth.facade.IRoleFacade;
import com.dragoninfo.dcuc.auth.power.facade.IAppFunInfoFacade;
import com.dragoninfo.dcuc.auth.sub.dto.AuthUserDTO;
import com.dragoninfo.dcuc.auth.sub.facade.IAuthUserInfoFacade;
import com.dragoninfo.dcuc.authweb.restcontroller.api.authservice.v3.vo.AppFunInfoApiVo;
import com.dragoninfo.dcuc.authweb.restcontroller.api.controller.pojo.FunAppovalParamVo;
import com.dragoninfo.dcuc.authweb.restcontroller.api.controller.pojo.RoleApiParamVo;
import com.dragoninfo.dcuc.authweb.restcontroller.api.controller.pojo.UpdataAppFunVO;
import com.dragoninfo.duceap.core.enums.ResultEnum;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

/**
 * @author Administrator
 * @date 2018/10/29
 */
@Api(tags = {"权限管理接口"})
@RestController
@RequestMapping(value = "/api")
public class AuthApiController {

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private IRoleFacade roleFacade;
    @Autowired
    private IApprovalFacade iApprovalFacade;
    @Autowired
    private IApplyInfoFacade applyInfoFacade;
    @Autowired
    private IAppFunInfoFacade iAppFunInfoFacade;
    @Autowired
    private IAuthUserInfoFacade authUserInfoFacade;

    @ApiOperation(value = "获取用户权限")
    @ApiImplicitParams({@ApiImplicitParam(name = "userId", value = "用户ID"),
            @ApiImplicitParam(name = "appCode", value = "应用代码")})
    @GetMapping(value = "/auth/getUserAuths")
    @ResponseBody
    public List<String> getAuthByUserIdAndAppCode(@RequestParam(value = "userId", required = true) String userId, @RequestParam(value = "appCode", required = true) String appCode) {
        return roleFacade.getAuthByUserIdAndAppCode(userId, appCode);
    }

    @ApiOperation(value = "获取应用权限")
    @ApiImplicitParams({@ApiImplicitParam(paramType = "path", name = "userId", value = "用户ID"),
            @ApiImplicitParam(paramType = "path", name = "appCode", value = "应用代码")})
    @GetMapping(value = "/auth/user/{userId}/appCode/{appCode}")
    @ResponseBody
    public List<Map<String, String>> getAppAuthByUserIdAndAppCode(@PathVariable(value = "userId") String userId, @PathVariable(value = "appCode") String appCode) {
        return roleFacade.getAppAuthByUserIdAndAppCode(userId, appCode);
    }

    /**
     * 获取所有应用
     *
     * @return
     */
    @ApiOperation(value = "获取所有应用")
    @GetMapping(value = "/auth/applications")
    @ResponseBody
    public List<ApplyInfo> applications() {
        return applyInfoFacade.getAllList();
    }

    @ApiOperation(value = "获取角色")
    @ApiImplicitParams({@ApiImplicitParam(name = "appCode", value = "应用代码", required = true, example = "11111"),
            @ApiImplicitParam(name = "userId", value = "用户身份证号", required = true)})
    @GetMapping(value = "/roles/{appCode}/{userId}")
    @ResponseBody
    public ResponseDTO getRolesByAppCodeAndUserId(@PathVariable(value = "appCode") String appCode, @PathVariable(value = "userId") String userId) {
        return roleFacade.getRolesByAppCodeAndUserId(appCode, userId);
    }

    /**
     * 动态授权申请
     *
     * @param appCode        应用代码
     * @param roleCode       角色代码
     * @param userIdCard     申请人身份证
     * @param approvalIdCard 审批人身份证
     * @param reason         申请事由
     * @return
     */
    @ApiOperation(value = "动态授权申请")
    @ApiImplicitParams({@ApiImplicitParam(name = "appCode", value = "应用代码", required = true
            , example = "11111"), @ApiImplicitParam(name = "roleCode", value = "角色代码", required = true
            , example = "11111"), @ApiImplicitParam(name = "userIdCard", value = "用户身份证号", required = true
            , example = "340111197810194754"), @ApiImplicitParam(name = "approvalIdCard", value = "审批人身份证", required = true
            , example = "340111197810194754"), @ApiImplicitParam(name = "reason", value = "申请事由", required = true
            , example = "340111197810194754")})
    @GetMapping(value = "/auth/apply")
    @ResponseBody
    public ResponseDTO proposer(
            @RequestParam(value = "appCode", required = true) String appCode,
            @RequestParam(value = "roleCode", required = true) String roleCode,
            @RequestParam(value = "userIdCard", required = true) String userIdCard,
            @RequestParam(value = "approvalIdCard", required = true) String approvalIdCard,
            @RequestParam(value = "reason", required = true) String reason) {
//        http://172.16.105.196:8888/dcuc/api/auth/apply?appCode=YHZX0000000000000001&roleCode=MANAGER_APP&userIdCard=000000000000000001&approvalIdCard=000000000000000001&reason=111
        return iApprovalFacade.authApply(appCode, roleCode, userIdCard, approvalIdCard, reason);
    }

    /**
     * 动态授权的角色功能获取
     *
     * @param appCode    应用代码
     * @param roleCode   角色代码
     * @param userIdCard 申请批人身份证
     * @return
     */
    @ApiOperation(value = "动态授权的角色功能获取")
    @ApiImplicitParams({@ApiImplicitParam(name = "appCode", value = "应用代码", required = true
            , example = "11111"), @ApiImplicitParam(name = "roleCode", value = "角色代码", required = true
            , example = "11111"), @ApiImplicitParam(name = "userIdCard", value = "用户身份证号", required = true
            , example = "340111197810194754")})
    @GetMapping(value = "/auth/applyResult")
    @ResponseBody
    public ResponseDTO getProposerFuns(
            @RequestParam(value = "appCode", required = true) String appCode,
            @RequestParam(value = "roleCode", required = true) String roleCode,
            @RequestParam(value = "userIdCard", required = true) String userIdCard) {
        ApprovalDto approvalDto = new ApprovalDto();
        approvalDto.setAppCode(appCode);
        approvalDto.setRoleCode(roleCode);
        approvalDto.setUserIdCard(userIdCard);
//        http://172.16.105.196:8888/dcuc/api/auth/applyResult?appCode=YHZX0000000000000001&roleCode=MANAGER_APP&userIdCard=000000000000000001
        return iApprovalFacade.applyResult(approvalDto);
    }

    /**
     * 1.1
     * 授权管理服务 获取用户菜单权限列表
     *
     * @param appCode
     * @return
     */
    @ApiOperation(value = "获取用户菜单权限列表")
    @ApiImplicitParams({@ApiImplicitParam(paramType = "path", name = "appCode", value = "应用代码", required = true
            , example = "11111")})
    @GetMapping(value = "/getResources/{appCode}")
    @ResponseBody
    public Result getMenus(@PathVariable(value = "appCode") String appCode) {
        try {
            RoleApiDto dto = new RoleApiDto();
            dto.setAppCode(appCode);
            List<AppFunInfoDTO> menus = roleFacade.getMenus(dto);
            List<AppFunInfoApiVo> vos = Optional.ofNullable(menus)
                    .orElse(new ArrayList<>())
                    .stream()
                    .map(item -> {
                        AppFunInfoApiVo vo = new AppFunInfoApiVo();
                        vo.setName(item.getName());
                        vo.setCode(item.getCode());
                        vo.setPid(item.getParentId());
                        vo.setUrl(item.getUrl());
                        vo.setSort(item.getSeq());
                        return vo;
                    }).collect(Collectors.toList());
            return Result.success(vos);
        } catch (Exception e) {
            return Result.fail("获取失败");
        }
    }

    /**
     * 1.2
     * 推送应用功能资源
     *
     * @return
     */
    @ApiOperation(value = "推送应用功能资源")
    @ApiImplicitParams({@ApiImplicitParam(name = "UpdataAppFunVO", value = "推送应用功能资源")})
    @PostMapping(value = "/setResources")
    @ResponseBody
    public Result updateAppFun(@RequestBody UpdataAppFunVO updataAppFunVO) {
        AppResourcesDto appResourcesDto = new AppResourcesDto(updataAppFunVO.getAppCode(), updataAppFunVO.getResources());
        try {
            ResponseDTO responseDTO = iAppFunInfoFacade.updateAppFun(appResourcesDto);
            if (ResponseDTO.FAIL_CODE.equals(responseDTO.getStatusCode())) {
                return Result.fail(responseDTO.getMessage());
            }
            return Result.success();
        } catch (Exception e) {
            logger.error("资源更新失败[{}]", e.getMessage());
            return Result.fail("资源更新失败", e.getMessage());
        }
    }

    /**
     * 1.3
     * 授权管理服务 获取用户角色权限列表
     *
     * @return
     */
    @ApiOperation(value = "获取用户角色权限列表")
    @ApiImplicitParams({@ApiImplicitParam(name = "RoleApiParamVo", value = "用户角色权限")})
    @PostMapping(value = "/getRoles")
    @ResponseBody
    public Result getRoles(@RequestBody RoleApiParamVo vo) {
        if (StringUtils.isEmpty(vo.getAppCode())) {
            return Result.fail("appCode不能为空");
        }
        try {
            RoleApiDto dto = new RoleApiDto(vo.getAppCode(), vo.getIdcard(), vo.getRoleCode());
            return Result.success(roleFacade.getRoleByAppCodeAndUserId(dto));
        } catch (Exception e) {
            return Result.fail(ResultEnum.FAIL.getKey(), e.getMessage());
        }
    }


    /**
     * 1.4
     * 动态授权
     *
     * @param vo
     * @return
     */
    @ApiOperation(value = "动态授权")
    @ApiImplicitParams({@ApiImplicitParam(name = "FunAppovalParamVo", value = "动态授权")})
    @RequestMapping("/pushData")
    @ResponseBody
    public Result pushData(@RequestBody FunAppovalParamVo vo) {
        String appCode = vo.getAppCode();
        String roleCode = vo.getRoleCode();
        String userIdCard = vo.getProposerIdcard();
        String approvalIdCard = vo.getApprpvalIdcard();
        String reason = vo.getProposerReason();
        if (StringUtils.isEmpty(appCode) || StringUtils.isEmpty(roleCode)
                || StringUtils.isEmpty(approvalIdCard) || StringUtils.isEmpty(userIdCard)
                || StringUtils.isEmpty(vo.getVerifiy()) || StringUtils.isEmpty(vo.getProposerOrg())) {
            return Result.fail("必填参数不能为空");
        }
        AuthUserDTO proposerUserInfo = authUserInfoFacade.findByIdcard(userIdCard);
        if (proposerUserInfo == null || !proposerUserInfo.getOrgId().equals(vo.getProposerOrg())) {
            return Result.fail("申请人不存在或机构错误");
        }
        ResponseDTO responseDTO = iApprovalFacade.authApply(appCode, roleCode, userIdCard, approvalIdCard, reason);
        if (ResponseDTO.FAIL_CODE.equals(responseDTO.getStatusCode())) {
            return Result.fail(responseDTO.getMessage());
        }
        return Result.success();
    }


    /**
     * 1.5
     * 获取权限列表成功
     *
     * @return
     */
    @ApiOperation(value = "获取权限列表")
    @ApiImplicitParams({@ApiImplicitParam(name = "RoleApiParamVo", value = "获取权限列表参数")})
    @PostMapping(value = "/getRoleRight")
    @ResponseBody
    public Result getRoleRight(@RequestBody RoleApiParamVo vo) {

        if (StringUtils.isEmpty(vo.getAppCode())) {
            return Result.fail("appCode不能为空");
        }
        try {
            RoleApiDto dto = new RoleApiDto(vo.getAppCode(), vo.getIdcard(), vo.getRoleCode());
            return Result.success(roleFacade.getRoleRight(dto));
        } catch (Exception e) {

            return Result.fail("获取失败");
        }
    }

    /**
     * 公共资源
     * 根据身份证ID找到这个赋予的应用
     *
     * @param idcard
     * @return
     */
    @ApiOperation(value = "根据身份证ID找到这个赋予的应用")
    @ApiImplicitParams({@ApiImplicitParam(paramType = "path", name = "idcard", value = "身份证号", required = true
            , example = "340102192904207238")})
    @GetMapping(value = "/apps/{idcard}")
    @ResponseBody
    public Result appListByIdcard(@PathVariable("idcard") String idcard) {
        try {
            return Result.success(roleFacade.appListByIdcard(idcard));
        } catch (Exception e) {
            return Result.fail("获取失败");
        }
    }


}
