package com.dragoninfo.dcuc.authweb.restcontroller.auth;

import com.dragoninfo.dcuc.auth.auth.entity.AppFunInfo;
import com.dragoninfo.dcuc.auth.auth.entity.FunEvent;
import com.dragoninfo.dcuc.auth.power.facade.IAppFunInfoFacade;
import com.dragoninfo.dcuc.auth.power.facade.IFunEventFacade;
import com.dragoninfo.dcuc.auth.auth.vo.AppFunInfoVo;
import com.dragoninfo.dcuc.authweb.restcontroller.auth.vo.FunEventVo;
import com.dragoninfo.duceap.core.response.Result;
import com.dragonsoft.duceap.base.entity.http.ResponseDTO;
import com.dragonsoft.duceap.base.entity.http.ResponseStatus;
import com.dragonsoft.duceap.base.entity.search.SearchDTO;
import com.dragonsoft.duceap.commons.util.string.StringUtils;
import com.dragonsoft.duceap.core.search.Searchable;
import com.dragonsoft.duceap.core.search.filter.Condition;
import com.dragonsoft.duceap.web.annotation.Permission;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

@Api(tags = {"授权模块-事件管理接口"})
@RestController
@RequestMapping(value = "authsvr/v2/funevent")
public class FunEventController {

    public static Logger logger = LoggerFactory.getLogger(FunEventController.class);

    @Autowired
    private IFunEventFacade iFunEventFacade;

    @Autowired
    private IAppFunInfoFacade iAppFunInfoFacade;


    /**
     * 菜单列表
     * @param searchDTO
     * @return
     */
    @ApiOperation(value = "根据功能菜单id获取菜单的功能事件列表")
    @ApiImplicitParam(name = "searchable", value = "查询条件,pkMenuId可为空")
    @GetMapping(value = "/menuList")
    public Result<List<FunEventVo>> menuList(SearchDTO searchDTO) {
        Searchable searchable = Searchable.toSearchable(searchDTO);
        Condition condition = searchable.getSearchFilterByKey("pkMenuId_eq");
        List<FunEventVo> resultList = new ArrayList<>();
        String pkMenuId = null;
        if(null !=condition) pkMenuId = (String) condition.getValue();
        List<FunEvent> funEvents = iFunEventFacade.getMenuData(pkMenuId, searchDTO);
        for (FunEvent funEvent : funEvents) {
            FunEventVo vo = new FunEventVo();
            BeanUtils.copyProperties(funEvent,vo);
            resultList.add(vo);
        }
        return Result.success(resultList);
    }

    /**
     *
     * @param appFunInfoVo
     * @return
     */
    @ApiOperation(value = "保存应用菜单")
    @Permission(value = "power_config")
    @PutMapping(value = "/funEvent",produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Result updateFunEvent(@RequestBody AppFunInfoVo appFunInfoVo) {
        Result result;
//        String menuId = request.getParameter("menuId");
        String id = appFunInfoVo.getId();
        ResponseDTO responseDTO;
        String message;
        AppFunInfo appFunInfo = new AppFunInfo();
        BeanUtils.copyProperties(appFunInfoVo,appFunInfo);
        if (StringUtils.isBlank(id)) {
            message = "保存";
            //id设置为null,避免detached entity passed to persist 异常
            appFunInfo.setId(null);
            responseDTO = iAppFunInfoFacade.save(appFunInfo);
        } else {
            message = "更新";
            appFunInfo.setId(id);
            responseDTO = iAppFunInfoFacade.update(appFunInfo);
        }
        if(ResponseStatus.SUCCESS_CODE.equals(responseDTO.getStatusCode())){
            result = Result.success(message + "成功");
        }else {
            result = Result.fail(message+"失败");
        }
        return result;
    }

    @ApiOperation(value = "删除应用菜单,如果(应用菜单)该节点下有子节点也进行删除")
    @ApiImplicitParams({@ApiImplicitParam(name = "deletedId", value = "菜单id"),@ApiImplicitParam(name = "appId", value = "应用id")})
    @Permission(value = "power_config")
    @DeleteMapping(value = "/menu",produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public Result deleteMenu(@RequestParam("deletedId") String deletedId,@RequestParam("appId") String appId) {
        Result result;
        try {
            if (deletedId != null) {
                iAppFunInfoFacade.deleteMenuAll(deletedId, appId);
            }
            result = Result.success("删除成功");
        } catch (Exception e) {
            result = Result.fail("删除异常");
            logger.error("删除异常", e);
        }
        return result;
    }
}